--------------------------------------------------------------
--	InterruptBar by Kollektiv. Modified by Wardz (Felesake)
--------------------------------------------------------------

-- Default abilities to track for
local defaultAbilities = {
	[57994] = 12,	[6552]  = 15,	[47528]  = 15,
	[1766]  = 15,	[96231] = 15,	[116705] = 15,
	[80965] = 15,	[49576] = 20,	[8177]   = 22,
	[34490] = 24,	[2139]  = 24,	[19647]  = 24,
	[47482] = 30,	[15487] = 45,	[19503]  = 30,
	[23920] = 25,
}

-- Spells that resets other spells
local spellResets = {
	-- Cold Snap
	[11958] = {45438,122,120,},
	-- Preparation
	[14185] = {2983,1856,5277,51722,},
	-- Call of the Elements
	[108285] = {108269,8177,108270,108273,51485,8143,},
}

-- Redirect these spells to one icon
local spellRedirects = {
	[19647] = {115781,132409,103135,24259,115782},
	[102060] = 6552,
	[147362] = 34490,
	[80965] = {80964,106839},
	[19263] = {65871,110617,110618},
	[16979] = 49376,
	[9005] = 102546,
	[339] = {113770,19975},
	[113004] = 113056,
	[19185] = 64803,
	[126451] = 122242,
	[5246] = 20511,
	[107570] = 132169,
	[25046] = {28730,50613,69179,80483,129597},
	[8177] = {8178,65989,34079,89523},
}

-- Locals
local IB = CreateFrame('Frame')
local bar,IsInBattleGround,barText,_
local btns = {}
local lastUpdate = 0
local intvUpdate = 0
local BackdropLayout = { bgFile = "Interface\\ChatFrame\\ChatFrameBackground", insets = { left = 0, right = 0, top = 0, bottom = 0 } }
local _,_,spellicon
local tNormal = 'Interface\\AddOns\\InterruptBar2\\media\\Normal.tga'
local tGloss = 'Interface\\AddOns\\InterruptBar2\\media\\Gloss.tga'
local tBorder = 'Interface\\AddOns\\InterruptBar2\\media\\Border.tga'
local settingsDefault = {scale=1,hidden=false,lock=false,rows=1,texthide=false,textsize=18,textdeci=false,border=true,offsety=30,offsetx=30,alpha=false,nobg=false,msg=false,smart=false,abilities = defaultAbilities,message=false,textx=0,texty=0,animation=false,version='1.0.9'}

-- Cache frequently used globals in locals
local _G = _G
local pairs = pairs
local floor = floor
local GetTime = GetTime
local band = bit.band
local srcFlags = srcFlags
local COMBATLOG_OBJECT_REACTION_HOSTILE = COMBATLOG_OBJECT_REACTION_HOSTILE
local GetSpellInfo = GetSpellInfo
local strformat = string.format
local TFONT = STANDARD_TEXT_FONT
local player = UnitName('player')
local SUCCESS = 'SPELL_CAST_SUCCESS'
local AURASUCCESS = 'SPELL_AURA_APPLIED'
local SUMMON = 'SPELL_SUMON'
local MISSED = 'SPELL_MISSED'
local INTERRUPTED = 'SPELL_INTERRUPT'

-- Savedvariables
InterruptBar2DB = InterruptBar2DB or settingsDefault

-- Reset addon on update or broken table
function IB:UpdateFix()
	if ( InterruptBar2DB.version ~= '1.0.9' or not InterruptBar2DB.version ) then
		wipe(InterruptBar2DB)
		InterruptBar2DB = settingsDefault
	elseif ( not InterruptBar2DB.abilities ) then
		InterruptBar2DB.abilities = defaultAbilities
	end
end

-- Text Cooldown (Mostly taken from Gladius, ty!)
function IB:UpdateTimerText(text, cooldown)
	if ( not InterruptBar2DB.texthide ) then
		local minutes = floor(cooldown / 60)

		if ( minutes > 0 ) then
			-- Show minutes and seconds
			local seconds = cooldown - minutes * 60

			text:SetTextColor(1, 1, 1, 1)
			text:SetFont(TFONT, InterruptBar2DB.textsize - 6 , 'OUTLINE')
			text:SetText(strformat('%sm \n%.0fs', minutes, seconds))
		else
			if ( cooldown > 5 ) then
				-- CD is higher than 5s and below 60s
				text:SetTextColor(1, 1, 0, 1)
				text:SetFont(TFONT, InterruptBar2DB.textsize - 3, 'OUTLINE')
				if ( not InterruptBar2DB.textdeci ) then
					-- Show text decimals
					text:SetText(strformat('%.0f', cooldown))
				else
					-- Hide text decimals
					text:SetText((strformat('%d', cooldown)))
				end
			else
				-- CD have less than 5s left
				text:SetTextColor(1, 0, 0, 1)
				text:SetFont(TFONT, InterruptBar2DB.textsize, 'OUTLINE')
				if ( cooldown == 0 ) then
					-- CD finished, hide text
					text:SetText('')
				else
					if ( not InterruptBar2DB.textdeci ) then
						-- Show text decimals
						text:SetText(strformat('%.1f', cooldown))
					else
						-- Hide text decimals
						text:SetText(strformat('%d', cooldown))
					end
				end
			end
		end
	end
end

-- Activate button received
function IB:Activate(btn,start)
	-- Restart if the spell already is being tracked
	if ( btn.active ) then
		IB:Deactivate(btn,start)
		IB:Activate(btn,start)
	end

	-- Show if transparent
	btn:SetAlpha(1)
	bar:Show() -- Necessary

	-- Show if hidden
	if ( InterruptBar2DB.hidden ) then
		btn:Show()
	end

	if ( InterruptBar2DB.animation ) then
		UIFrameFadeIn(btn, 1, 0, 1)
	end

	-- Set cooldown
	local cooldown = btn.duration - ( GetTime() - start )
	
	-- Start text timer
	if ( not InterruptBar2DB.texthide ) then
		IB:UpdateTimerText(btn.text, cooldown) -- Set text CD
		btn.text:SetAlpha(1) -- Show text
	end

	btn.cd:Show()
	btn.cd:SetCooldown(GetTime(), cooldown) -- Set spiral CD     ---- BTN.DURATION original

	-- OnUpdate
	if ( cooldown > 0 ) then
		btn:SetScript('OnUpdate', function(self, elapsed)
			lastUpdate = lastUpdate + elapsed
			if ( lastUpdate > intvUpdate ) then
				cooldown = cooldown - elapsed
				if ( cooldown <= 0 ) then
					-- CD is finished
					IB:Deactivate(self)
					if ( InterruptBar2DB.message and UnitInParty('player') ) then
						local abilityName = GetSpellInfo(btn.name)
						SendChatMessage(abilityName..' is off cooldown!', 'PARTY')
					end
				else
					if ( not InterruptBar2DB.texthide ) then
						-- Update text if enabled
						IB:UpdateTimerText(self.text, cooldown)
					end
				end
			end
			lastUpdate = 0
		end)
	end

	btn.active = true

	-- Smart Mode
	if ( InterruptBar2DB.smart and InterruptBar2DB.hidden ) then
		IB:UpdatePositions()
	end
end

-- Deactivate button received
function IB:Deactivate(btn,start)

	-- Fade Out
	if ( InterruptBar2DB.animation and InterruptBar2DB.hidden ) then
		UIFrameFadeOut(btn, 1, 1, 0)
	end

	-- Hide
	if ( InterruptBar2DB.hidden and not InterruptBar2DB.animation ) then
		btn:Hide()
	end

	-- Transparent
	if ( not InterruptBar2DB.hidden ) then
		if ( InterruptBar2DB.alpha ) then
			btn:SetAlpha(0.30)
		end
	end

	-- Hide text CD
	if ( not InterruptBar2DB.texthide ) then
		btn.text:SetAlpha(0)
	end

	btn:SetScript('OnUpdate', nil)
	btn.cd:SetCooldown(0, 0) -- For animation
	btn.active = false

	-- Smart Mode
	if ( InterruptBar2DB.smart and InterruptBar2DB.hidden ) then
		IB:UpdatePositions()
	end
end

-- Creates frame for every spell in table
function IB:CreateIcon(k, v)
	local btn = CreateFrame('Frame', nil, bar)

	if ( InterruptBar2DB.border ) then
		-- Background
		local textureBG = btn:CreateTexture(nil, 'OVERLAY')
		textureBG:SetAllPoints(btn)
		textureBG:SetTexture(tNormal)
		textureBG:SetTexCoord(.08, .92, .08, .92)

		-- Overlay / Gloss
		local textureGloss = btn:CreateTexture(nil, 'BORDER')
		textureGloss:SetAllPoints(btn)
		textureGloss:SetTexture(tGloss)
		textureGloss:SetTexCoord(.08, .92, .08, .92)
		textureGloss:SetAlpha(.25)

		-- Border
		local textureBorder = btn:CreateTexture(nil, 'BORDER')
		textureBorder:SetAllPoints(btn)
		textureBorder:SetTexture(tBorder)
		textureBorder:SetTexCoord(.08, .92, .08, .92)
		textureBorder:SetBlendMode('ADD')
		-- Hide this frame in transparent mode, as it looks really ugly
		if ( InterruptBar2DB.alpha ) then textureBorder:SetAlpha(0) end
	end

	-- Cooldown Spiral
	local cd = CreateFrame('Cooldown', nil, btn)
	cd:SetAllPoints(btn)
	cd:SetFrameStrata('MEDIUM')
	cd.noomnicc = true
	cd.noOCC = true
	cd.noCooldownCount = true
	cd:Hide()

	-- Spell icon texture
	local texture = btn:CreateTexture(nil, 'BACKGROUND')
	texture:SetAllPoints(btn)
	_,_,spellicon = GetSpellInfo(k)
	btn.spellicon = spellicon
	texture:SetTexture(spellicon)
	texture:SetTexCoord(.07,.9,.07,.90)

	-- Text Timer
	if ( not InterruptBar2DB.texthide ) then
		local text = cd:CreateFontString(nil, 'ARTWORK')
		text:ClearAllPoints()
		text:SetShadowOffset(1, -1)
		text:SetShadowColor(0, 0, 0, 1)
		text:SetAlpha(0)

		text:SetPoint('CENTER', btn, 'CENTER', InterruptBar2DB.textx, InterruptBar2DB.texty)
		btn.text = text
	end

	btn:SetWidth(30)
	btn:SetHeight(30)
	btn:SetFrameStrata('LOW')
	btn.texture = texture
	btn.duration = v
	btn.name = k
	btn.cd = cd

	return btn
end

-- Creates icon on bar for every spell found in local abilities
function IB:AddIcons()
	local offset = InterruptBar2DB.offsetx
	local height = InterruptBar2DB.rows
	local y = -45
	local x = 0
	local count = 0
	local abilities = InterruptBar2DB.abilities

	for k, v in pairs(abilities) do
		if ( count > height ) then
			y = -45
			x = x + offset
			count = 0
		end

		local btn = IB:CreateIcon(k,v)
		btn:SetPoint('CENTER', bar, 'CENTER', x, y)
		btns[k] = btn

		y = y + InterruptBar2DB.offsety
		count = count + 1
	end
end

-- Smart Mode
function IB:UpdatePositions()
	local offset = InterruptBar2DB.offsetx
	local height = InterruptBar2DB.rows
	local y = -45
	local x = 0
	local count = 0

	for _,btn in pairs(btns) do
		if ( btn.active ) then
			if ( count > height ) then
				y = -45
				x = x + offset
				count = 0
			end

			btn:SetPoint('CENTER', bar, 'CENTER', x, y)

			y = y + InterruptBar2DB.offsety
			count = count + 1
		end
	end
end

-- Save frame position
function IB:SavePosition()
	-- Get current frame position
	local point, _, relativePoint, xOfs, yOfs = bar:GetPoint()

	-- Create new savedvariable if not exists
	if ( not InterruptBar2DB.Position ) then
		InterruptBar2DB.Position = {}
	end

	-- Save to savedvariable
	InterruptBar2DB.Position.point = point
	InterruptBar2DB.Position.relativePoint = relativePoint
	InterruptBar2DB.Position.xOfs = xOfs
	InterruptBar2DB.Position.yOfs = yOfs
end

-- Load frame position
function IB:LoadPosition()
	if ( InterruptBar2DB.Position ) then
		-- If savedvariable (user-placed) positon found then
		bar:SetPoint(InterruptBar2DB.Position.point,UIParent,InterruptBar2DB.Position.relativePoint,InterruptBar2DB.Position.xOfs,InterruptBar2DB.Position.yOfs)
	else
		-- Saved position not found
		bar:SetPoint('CENTER', UIParent, 'CENTER')
	end
end

-- Create frame to hold all btn frames
function IB:CreateBar()
	if ( not IsInBattleGround ) then
		bar = CreateFrame('Frame', nil, UIParent)
		bar:SetMovable(1)
		bar:SetClampedToScreen(1)
		bar:SetScale(InterruptBar2DB.scale)
		bar:SetScript('OnMouseDown', function() bar:StartMoving() end)
		bar:SetScript('OnMouseUp', function(self,button) if ( button == 'RightButton' ) then InterruptBar2DB.lock = true IB:Restart() else bar:StopMovingOrSizing() IB:SavePosition() end end)

		bar:SetWidth(200)
		bar:SetHeight(15)

		-- text
		barText = bar:CreateFontString(nil, 'ARTWORK')
		barText:SetPoint('TOP', bar, 'TOP')
		barText:SetFont(TFONT, 10 , 'OUTLINE')
		barText:SetText('Left click - move | Right Click - Lock')
		bar:SetBackdrop(BackdropLayout)
		bar:SetBackdropColor(0, 0, 0, .8)
		bar:Show()

		IB:AddIcons()
		IB:LoadPosition()
		IB:UpdateFrame()
	end
end

-- Combat log unfiltered
function IB:COMBAT_EVENT(...)
	-- Define vararg
	local _, eventType, _, _, srcName, srcFlags, _, _, destName, _, _, id = ...

	if ( band(srcFlags, COMBATLOG_OBJECT_REACTION_HOSTILE) ~= 0 )  then		
			-- Reduce CD on counterspell when interrupted on cast, due to setbonus
			if ( id == 2139 and btns[2139] ) then
				if ( eventType == INTERRUPTED ) then
					for i = 1, 5 do
						if ( destName == player or destName == UnitName('party'..i) ) then
							local btn = btns[2139]
							btn.duration = btn.duration - 4
							local start = start or GetTime()

							IB:Activate(btn,start)
							return
						end
					end
				end
			end

			-- Keep specific spells under one single icon
			for k,v in pairs(spellRedirects) do
				if id == v then
					local btn = btns[k]
					if ( btn ) then
						local start = start or GetTime()
						IB:Activate(btn,start)
						return
					end
				end
			end

		if ( eventType == SUCCESS or eventType == AURASUCCESS or eventType == SUMMON or eventType == MISSED and btns[id] or spellResets[id] ) then

			-- Reset abilities
			if ( spellResets[id] ) then
				for _,v in pairs(spellResets[id]) do
					local btn = btns[v]
					if ( btn ) then
						IB:Deactivate(btn)
					end
				end
			end

			-- Activate
			local btn = btns[id]
			if ( btn ) then
				local start = start or GetTime()
				IB:Activate(btn,start) -- Activate button
				if ( InterruptBar2DB.message ) and ( UnitInParty('player') ) then
					-- Announce in party
					local spellNameText = GetSpellInfo(id)
					SendChatMessage(spellNameText..' used by '..srcName..'!', 'PARTY')
				end
			end
		end
	end
end

-- Update settings to frames
function IB:UpdateFrame()
	bar:Show()
	-- Lock
	if ( InterruptBar2DB.lock ) then
		bar:EnableMouse(0)
		barText:Hide()
		bar:SetBackdropColor(0, 0, 0, 0)
	else
		bar:EnableMouse(1)
		barText:Show()
		bar:SetBackdropColor(0, 0, 0, .3)
	end

	for _,btn in pairs(btns) do
		-- Hide
		if ( InterruptBar2DB.hidden ) then
			btn:Hide()
		else
			btn:Show()
		end

		-- Transparent
		if ( InterruptBar2DB.alpha ) and ( not InterruptBar2DB.hidden ) then
			btn:SetAlpha(0.3)
		end
	end
end

-- Recreate frames
function IB:Restart()
	for _,btn in pairs(btns) do
		IB:Deactivate(btn)
		btn:Hide()
	end
	--btns = {} 
	wipe(btns)
	bar:Hide()
	IB:CreateBar()
end

------------------------------------------------------------------------------------------
-- OPTIONS
------------------------------------------------------------------------------------------
-- Create slider
function IB:CreateSlider(text, parent, low, high, step, value)
	local name = 'IB2:'..text
	local slider = CreateFrame('Slider', name, parent, 'OptionsSliderTemplate')
	slider:SetWidth(145)
	slider:SetMinMaxValues(low, high)
	slider:SetValueStep(step)
	slider:SetObeyStepOnDrag(1)
	slider:SetValue(value)
	_G[name..'Text']:SetText(text)
	_G[name..'Low']:SetText(low)
	_G[name..'High']:SetText(high)

	return slider
end

-- Create button
function IB:CreateButton(text, parent)
	local button = CreateFrame('Button', nil, parent, 'UIPanelButtonTemplate')
	button:SetWidth(85)
	button:SetHeight(30)
	button.text = button:CreateFontString(nil, 'ARTWORK', 'GameFontNormal')
	button.text:SetPoint('CENTER', button, 'CENTER', 1, -1)
	button.text:SetText(text)

	return button
end

-- Create checkbox
function IB:CreateCheckBox(text,parent,value)
	local name = 'IB:'..text
	local checkbox = CreateFrame('CheckButton', name, parent, 'ChatConfigCheckButtonTemplate')
	_G[checkbox:GetName()..'Text']:SetText(text)
	if value then checkbox:SetChecked(1) else checkbox:SetChecked(0) end

	return checkbox
end

-- Create text
function IB:CreateText(textmsg,parent,tsize)
	local text = parent:CreateFontString(nil, 'ARTWORK')
	text:SetFont(TFONT, tsize)
	text:SetText(textmsg)
	return text
end

-- Format user input
function IB:FormatSpellName(getSpell)
	getSpell = getSpell:gsub("(%a)(%a*)('*)(%a*)", function (a,b,c,d) return a:upper()..b:lower()..c..d:lower() end)
	getSpell = getSpell:gsub('(The)', string.lower)
	getSpell = getSpell:gsub('(Of)', string.lower)
	--GetSpellLink(getSpell):match('spell:(%d+)')
	for i = 1, 100000 do
		if getSpell == GetSpellInfo(i) then
			return i
		end
	end
end

local IB2Options = CreateFrame('Frame')
IB2Options.name = 'InterruptBar2'

-- Header Text Part 1
local T1 = IB:CreateText('InterruptBar2 Settings', IB2Options, 20)
T1:SetPoint('TOP', IB2Options, 'TOP', -190,-20)

-- Header Text Part 2
local T2 = IB:CreateText('v1.0.9', IB2Options, 10)
T2:SetPoint('TOP', IB2Options, 'TOP', -275,-42)

-- Test button
local BTN1 = IB:CreateButton('Test', IB2Options)
BTN1:SetPoint('BOTTOM', 250, 13)
BTN1:SetScript('OnClick', function()
	for _,btn in pairs(btns) do
		local start = start or GetTime()
		IB:Deactivate(btn)
		IB:Activate(btn,start)
	end
end)

-- Reset Button
IB2Options.default = function()
	wipe(InterruptBar2DB)
	InterruptBar2DB = settingsDefault
	IB:Restart()
	IB:SetValues()
end

IB2Options.refresh = function()
	IB:SetValues()
end

-- Text 3
local T3 = IB:CreateText('Distance between icons', IB2Options, 12)
T3:SetPoint('TOP', IB2Options, 'TOP', -215, -70)

-- Frame offset X
local SL1 = IB:CreateSlider('Offset X', IB2Options, 30, 100, 1, InterruptBar2DB.offsetx)
SL1:SetPoint('LEFT', 30, 165)
SL1:SetScript('OnValueChanged', function(self, value)
	_G[self:GetName() .. 'Text']:SetText('Offset X' .. ' (' .. value .. ')')
	InterruptBar2DB.offsetx = tonumber(value)
	IB:Restart()
end)

-- Frame offset Y
local SL2 = IB:CreateSlider('Offset Y', SL1, 30, 100, 1, InterruptBar2DB.offsety)
SL2:SetPoint('LEFT', 175, 0)
SL2:SetScript('OnValueChanged', function(self, value)
	if ( InterruptBar2DB.rows > 0 ) then
		_G[self:GetName() .. 'Text']:SetText('Offset Y' .. ' (' .. value .. ')')
		InterruptBar2DB.offsety = tonumber(value)
		IB:Restart()
	else
		SL2:SetValue(InterruptBar2DB.offsety)
		print('IB2: Requires more than 1 row on bar to use this feature.')
	end
end)

-- Text 4
local T4 = IB:CreateText('Icon size and columns', IB2Options, 12)
T4:SetPoint('TOP', IB2Options, 'TOP', -215, -160)

-- Icon scale
local SL3 = IB:CreateSlider('Icon Scale', IB2Options, 0.1, 5, 0.1, InterruptBar2DB.scale)
SL3:SetPoint('LEFT', 30, 80)
SL3:SetScript('OnValueChanged', function(self, value)
	_G[self:GetName() .. 'Text']:SetText('Icon Scale' .. ' (' .. value .. ')')
	InterruptBar2DB.scale = tonumber(value)
	IB:Restart()
end)

-- Rows
local SL4 = IB:CreateSlider('Bar Rows', SL3, 0, 25, 1, InterruptBar2DB.rows)
SL4:SetPoint('LEFT', 175, 0)
SL4:SetScript('OnValueChanged', function(self, value)
	_G[self:GetName() .. 'Text']:SetText('Bar Rows' .. ' (' .. value .. ')')
	InterruptBar2DB.rows = tonumber(value)
	IB:Restart()
end)

-- Text 5
local T5 = IB:CreateText('Misc Settings', IB2Options, 12)
T5:SetPoint('TOP', IB2Options, 'TOP', -243, -250)

-- Lock Bar
local CB1 = IB:CreateCheckBox(' Lock Bar', IB2Options, InterruptBar2DB.lock)
CB1:SetPoint('LEFT', 40, 0)
CB1:SetScript('OnClick', function()
	InterruptBar2DB.lock = not InterruptBar2DB.lock
	IB:Restart()
end)

-- Border Textures
local CB2 = IB:CreateCheckBox(' Enable textures', CB1, InterruptBar2DB.border)
CB2:SetPoint('LEFT', 165, 0)
CB2:SetScript('OnClick', function()
	InterruptBar2DB.border = not InterruptBar2DB.border
	IB:Restart()
end)

-- Disable in BattleGrounds
local CB3 = IB:CreateCheckBox(' Disable in BG', IB2Options, InterruptBar2DB.nobg)
CB3:SetPoint('LEFT', 40, -30)
CB3:SetScript('OnClick', function()
	InterruptBar2DB.nobg = not InterruptBar2DB.nobg
	IB:Restart()
end)

-- Hide Mode
local CB4 = IB:CreateCheckBox(' Hide-Mode', IB2Options, InterruptBar2DB.hidden)
CB4:SetPoint('LEFT', 40, -60)
CB4:SetScript('OnClick', function()
	InterruptBar2DB.hidden = not InterruptBar2DB.hidden
	IB:Restart()
	if ( InterruptBar2DB.alpha ) and ( InterruptBar2DB.hidden ) then
		print('IB2: Transparent-Mode is not supposed to be used together with Hide-Mode')
	end
end)

-- Transparent Mode
local CB5 = IB:CreateCheckBox(' Transparent-Mode', CB4, InterruptBar2DB.alpha)
CB5:SetPoint('LEFT', 165, 0)
CB5:SetScript('OnClick', function()
	InterruptBar2DB.alpha = not InterruptBar2DB.alpha
	IB:Restart()
	if ( InterruptBar2DB.alpha ) and ( InterruptBar2DB.hidden ) then
		print('IB2: Transparent-Mode is not supposed to be used together with Hide-Mode')
	end
end)

-- Disable login message
local CB6 = IB:CreateCheckBox(' Hide login message', IB2Options, InterruptBar2DB.msg)
CB6:SetPoint('LEFT', 40, -90)
CB6:SetScript('OnClick', function()
	InterruptBar2DB.msg = not InterruptBar2DB.msg
	IB:Restart()
end)

-- Animations
local CB7 = IB:CreateCheckBox(' Fade icons out/in', CB6, InterruptBar2DB.animation)
CB7:SetPoint('LEFT', 165, 0)
CB7:SetScript('OnClick', function()
	InterruptBar2DB.animation = not InterruptBar2DB.animation
	IB:Restart()
end)

-- Smart Mode
local CB8 = IB:CreateCheckBox(' Smart-Mode', CB3, InterruptBar2DB.smart)
CB8:SetPoint('LEFT', 165, 0)
CB8:SetScript('OnClick', function()
	InterruptBar2DB.smart = not InterruptBar2DB.smart
	if ( InterruptBar2DB.smart ) and ( not InterruptBar2DB.hidden ) then
		print('IB2: Smart-Mode currently only works with Hide-Mode enabled as well.')
	end
	IB:Restart()
end)

local CB9 = IB:CreateCheckBox(' Announce interrupts in party', IB2Options, InterruptBar2DB.message)
CB9:SetPoint('LEFT', 40, -120)
CB9:SetScript('OnClick', function()
	InterruptBar2DB.message = not InterruptBar2DB.message
	IB:Restart()
end)

-- Text 6
local T6 = IB:CreateText('Add new spell', IB2Options, 12)
T6:SetPoint('LEFT', IB2Options, 'LEFT', 36, -170)

-- Text 7
local T7 = IB:CreateText('Spell Name or Spell ID', IB2Options, 9)
T7:SetPoint('LEFT', IB2Options, 'LEFT', 39, -190)

-- Spell EditBox
local EB1 = CreateFrame('EditBox', 'IB:EB1', IB2Options, 'InputBoxTemplate')
EB1:SetPoint('LEFT', 40, -210)
EB1:SetSize(150,50)
EB1:EnableMouse(1)
EB1:SetFont(TFONT, 10, 'OUTLINE')
EB1:SetAutoFocus(0)

local BTN3 = IB:CreateButton('Add', EB1)
BTN3:SetPoint('LEFT', 160, 0)
BTN3:SetScript('OnClick', function()
	local getSpell = EB1:GetText()
		
	if ( getSpell and getSpell ~= '' ) then
		local spellCD
		local spellName
		
		if ( tonumber(getSpell) ) then
			-- If above is true, then input is numbers only
			spellName = tonumber(getSpell) -- To number here or spell gets treated as string in table

			-- HACK: deterrence will put 6 sec CD due to charge, but is original 3 min
			if ( spellName == 19263 ) then
				spellCD = 180
			else
				spellCD = GetSpellBaseCooldown(tonumber(getSpell)) -- Get cooldown in miliseconds
				if ( spellCD ) then
					spellCD = spellCD / 1000
				else
					--spellCD = false
					print('IB2: Invalid input or spell.')
				end
			end
		else
			--[[Input is not numbers
				Convert spell name input to spell ID, because you can only use
				spell ID with GetSpellBaseCooldown and to get icons
			]]

			spellName = IB:FormatSpellName(getSpell)

			if ( spellName ) then
				-- HACK: deterrence will put 6 sec CD due to charge, but is original 3 min
				-- Check if there's more spells like this.
				if ( spellName == 19263 ) then
					spellCD = 180
				else
					spellCD = GetSpellBaseCooldown(spellName)

					if ( spellCD ) then
						spellCD = spellCD / 1000
					else
						-- Invalid spell ID sent to GetSpellBaseCooldown
						spellCD = false
						print('IB2: Invalid input or spell.')
					end
				end
			else
				-- Could not find valid spell name by ID
				spellName = false
				print('IB2: Invalid input or spell.')
			end
		end

		if ( spellName and spellCD ) then
			if ( not InterruptBar2DB.abilities[spellName] ) then
				-- Was able to get spell ID and CD, and spell does not exists already

				-- Insert to table
				InterruptBar2DB.abilities[spellName] = spellCD

				-- Restart
				IB:Restart()
				EB1:SetText('')
				print('IB2: '..spellName..' successfully added with cooldown: '..spellCD)
			else
				print('IB: Spell: '..spellName..' already exists.')
			end
		end
	end
end)

local BTN4 = IB:CreateButton('Delete', EB1)
BTN4:SetPoint('LEFT', 260, 0)
BTN4:SetScript('OnClick', function()
	local getSpell = EB1:GetText()
		
	if ( getSpell and getSpell ~= '' ) then
		local spellName
		
		if ( tonumber(getSpell) ) then
			-- If above is true, then input is numbers only
			spellName = tonumber(getSpell)
		else
			spellName = IB:FormatSpellName(getSpell)
		end

		if ( spellName ) then
			if ( InterruptBar2DB.abilities[spellName] ) then
			-- Was able to get spell ID and CD, and spell exists in table

			-- Insert to table
			InterruptBar2DB.abilities[spellName] = nil

			-- Restart
			IB:Restart()
			EB1:SetText('')
			print('IB2: '..spellName..' successfully removed.')
			else
				print('IB2: '..spellName..' does not exists.')
			end
		else
			-- could not get spellid
			print('IB2: Invalid spell name or ID.')
		end
	else
		-- User input does not match the allowed characters
		print('IB2: Invalid input')
	end
end)

-- Timer Settings TAB Frame
local IB2OptionsTimer = CreateFrame('Frame', nil, IB2Options)
IB2OptionsTimer.name = 'Timer Settings'
IB2OptionsTimer.parent = IB2Options.name

IB2OptionsTimer.refresh = function()
	IB:SetValues()
end

-- Header Text Part 1
local TimerT1 = IB:CreateText('InterruptBar2 Timer Settings', IB2OptionsTimer, 20)
TimerT1:SetPoint('TOP', IB2OptionsTimer, 'TOP', -162,-20)

-- Header Text Part 2
local TimerT2 = IB:CreateText('v1.0.9', IB2OptionsTimer, 10)
TimerT2:SetPoint('TOP', IB2OptionsTimer, 'TOP', -275,-42)

-- Test button
local TimerBTN1 = IB:CreateButton('Test', IB2OptionsTimer)
TimerBTN1:SetPoint('BOTTOM', 250, 13)
TimerBTN1:SetScript('OnClick', function()
	local start = start or GetTime()
	for _,btn in pairs(btns) do
		IB:Deactivate(btn,start)
		IB:Activate(btn,start)
	end
end)

-- Text 1
local TimerT1 = IB:CreateText('Font size', IB2OptionsTimer, 12)
TimerT1:SetPoint('TOP', IB2OptionsTimer, 'TOP', -258, -70)

-- Text Size
local TimerSL1 = IB:CreateSlider('Text Size', IB2OptionsTimer, 7, 35, 1, InterruptBar2DB.textsize)
TimerSL1:SetPoint('LEFT', 30, 165)
TimerSL1:SetScript('OnValueChanged', function(self, value)
	_G[self:GetName() .. 'Text']:SetText('Text Size' .. ' (' .. value .. ')')
	InterruptBar2DB.textsize = tonumber(value)
	IB:Restart()
end)

-- Text 2
local TimerT2 = IB:CreateText('Misc Settings', IB2OptionsTimer, 12)
TimerT2:SetPoint('TOP', IB2OptionsTimer, 'TOP', -245, -165)

-- Hide decimals
local TimerCB1 = IB:CreateCheckBox(' Hide decimals', IB2OptionsTimer, InterruptBar2DB.textdeci)
TimerCB1:SetPoint('TOP', IB2OptionsTimer, 'TOP', -257, -185)
TimerCB1:SetScript('OnClick', function()
	InterruptBar2DB.textdeci = not InterruptBar2DB.textdeci
	IB:Restart()
end)

-- Hide text
local TimerCB2 = IB:CreateCheckBox(' Hide text', TimerCB1, InterruptBar2DB.texthide)
TimerCB2:SetPoint('LEFT', 180, 0)
TimerCB2:SetScript('OnClick', function()
	InterruptBar2DB.texthide = not InterruptBar2DB.texthide
	for _,btn in pairs(btns) do
		btn:Hide()
		bar:Hide()
	end
	IB:CreateBar()
end)

-- Text 1
local TimerT3 = IB:CreateText('Text Position', IB2OptionsTimer, 12)
TimerT3:SetPoint('LEFT', IB2OptionsTimer, 'LEFT', 30, 40)

-- Text Offset X
local TimerSL2 = IB:CreateSlider('Text Offset X', IB2OptionsTimer, -200, 200, 1, InterruptBar2DB.textx)
TimerSL2:SetPoint('LEFT', 40, 0)
TimerSL2:SetScript('OnValueChanged', function(self, value)
	_G[self:GetName() .. 'Text']:SetText('Text Offset X' .. ' (' .. value .. ')')
	InterruptBar2DB.textx = tonumber(value)
	IB:Restart()
end)

-- Text Offset Y
local TimerSL3 = IB:CreateSlider('Text Offset Y', TimerSL2, -200, 200, 1, InterruptBar2DB.texty)
TimerSL3:SetPoint('LEFT', 175, 0)
TimerSL3:SetScript('OnValueChanged', function(self, value)
	_G[self:GetName() .. 'Text']:SetText('Text Offset Y' .. ' (' .. value .. ')')
	InterruptBar2DB.texty = tonumber(value)
	IB:Restart()
end)

function IB:SetValues()
	-- Remember changed settings for every loading screen
	-- Better way to do this?
	SL1:SetValue(InterruptBar2DB.offsetx)
	SL2:SetValue(InterruptBar2DB.offsety)
	SL3:SetValue(InterruptBar2DB.scale)
	SL4:SetValue(InterruptBar2DB.rows)
	if InterruptBar2DB.lock then CB1:SetChecked(1) else CB1:SetChecked(0) end
	if InterruptBar2DB.border then CB2:SetChecked(1) else CB2:SetChecked(0) end
	if InterruptBar2DB.nobg then CB3:SetChecked(1) else CB3:SetChecked(0) end
	if InterruptBar2DB.hidden then CB4:SetChecked(1) else CB4:SetChecked(0) end
	if InterruptBar2DB.alpha then CB5:SetChecked(1) else CB5:SetChecked(0) end
	if InterruptBar2DB.msg then CB6:SetChecked(1) else CB6:SetChecked(0) end
	if InterruptBar2DB.animation then CB7:SetChecked(1) else CB7:SetChecked(0) end
	if InterruptBar2DB.smart then CB8:SetChecked(1) else CB8:SetChecked(0) end
	if InterruptBar2DB.message then CB9:SetChecked(1) else CB9:SetChecked(0) end
	TimerSL1:SetValue(InterruptBar2DB.textsize)
	TimerSL2:SetValue(InterruptBar2DB.textx)
	TimerSL3:SetValue(InterruptBar2DB.texty)
	if InterruptBar2DB.textdeci then TimerCB1:SetChecked(1) else TimerCB1:SetChecked(0) end
	if InterruptBar2DB.texthide then TimerCB2:SetChecked(1) else TimerCB2:SetChecked(0) end
	EB1:SetText('')
end
------------------------------------------------------------------------------------------
------------------------------------------------------------------------------------------

-- On variables loaded
function IB:OnLoad()
	IB:RegisterEvent('PLAYER_ENTERING_WORLD')
	IB:RegisterEvent('PLAYER_ENTERING_BATTLEGROUND')
	IB:RegisterEvent('COMBAT_LOG_EVENT_UNFILTERED')

	InterruptBar2DB = InterruptBar2DB
	IB:UpdateFix()
	IB:CreateBar()

	InterfaceOptions_AddCategory(IB2Options)
	InterfaceOptions_AddCategory(IB2OptionsTimer)
	SlashCmdList['InterruptBar'] = function()  InterfaceOptionsFrame_OpenToCategory(IB2Options)InterfaceOptionsFrameOkay:Click('button', down)InterfaceOptionsFrame_OpenToCategory(IB2Options) end
	SLASH_InterruptBar1 = '/ib'
	SLASH_InterruptBar2 = '/ib2'
	SLASH_InterruptBar3 = '/interruptbar'
	SLASH_InterruptBar4 = '/interruptbar2'

	if ( not InterruptBar2DB.msg ) then
		ChatFrame1:AddMessage('InterruptBar2 loaded. Type /ib for options.', .3, .4, 1)
	end
end

-- Event Handler
IB:SetScript('OnEvent', function(self, event, ...)
	local _,instance = IsInInstance()
	-- Check if player is outside a battleground
	if ( instance == 'arena' or 'none' or 'party' or 'raid' ) then IsInBattleGround = false end

	-- Events
	if ( event == 'VARIABLES_LOADED' ) then
		IB:OnLoad()
	elseif ( event == 'PLAYER_ENTERING_WORLD' ) then
		if ( not IsInBattleGround ) then
			IB:UpdateFrame()
			IB:Restart()
		end
	elseif ( event == 'PLAYER_ENTERING_BATTLEGROUND' ) then
		if ( InterruptBar2DB.nobg ) then
			IsInBattleGround = true
			IB:Restart()
		end
	elseif ( event == 'COMBAT_LOG_EVENT_UNFILTERED' ) then
		if ( not IsInBattleGround ) then
			IB:COMBAT_EVENT(...)
		end
	end
end)

IB:RegisterEvent('VARIABLES_LOADED')